#*******************************************************************************
#**  This file is part of Qt Mocks.                                          ***
#**                                                                          ***
#**  Copyright (C) 2018, 2019, 2020, 2021                                    ***
#**  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
#**                                                                          ***
#**  This program is free software: you can redistribute it and/or modify it ***
#**  under the terms of the GNU Lesser General Public License as published   ***
#**  by the Free Software Foundation, either version 3 of the License, or    ***
#**  (at your option) any later version.                                     ***
#**                                                                          ***
#**  This program is distributed in the hope that it will be useful, but     ***
#**  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
#**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
#**  See the GNU Lesser General Public License for more details.             ***
#**                                                                          ***
#**  You should have received a copy of the GNU Lesser General Public License***
#**  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
#*******************************************************************************

#!/usr/bin/env bash
shopt -s -o nounset
shopt -s -o pipefail
shopt -s -o errexit
shopt -s extglob

# Program Variables
#___________________________________________________________________________________________________
declare -r scriptName=${0##*/}
declare -r path="$( cd "$(dirname "$0")" >/dev/null 2>&1 ; pwd -P )"
declare -r options="b:hk:"
declare optionResult=""
declare -r projectName="qt-mocks"
declare version=""
declare keyId=""
declare -r debBuildDirectoryPath="$HOME/debbuild"
declare tarballFileName=""
declare tarballOrigFileName=""
# licence in copyright file (apache|artistic|bsd|gpl|gpl2|gpl3|isc|lgpl|lgpl2|lgpl3|mit|custom)
declare -r licence="lgpl3"
# single, indep (arch-independent), library, python
declare -r packageclass="library"

# Program Argument Validation
#___________________________________________________________________________________________________
while getopts "${options}" optionResult
do
	case ${optionResult} in
	b)  # Set Branch Name
		branch="$OPTARG"
		;;
	h)  # Show help
		printf "usage: ${scriptName} [options]\n"
		printf "Options:\n"
		printf "  -b <name>       Sets the name of the branch to package.\n"
		printf "  -k <id>         Sets the ID of the sign key.\n"
		exit 0
		;;
	k)  # Set Key Sign ID
		keyId="$OPTARG"
		;;
	\?)
		exit 2
		;;
	*)
		printf "Unknown option: ${optionResult}\n" 1>&2
		exit 2
		;;
	esac
done

if [[ -z "${branch}" ]]
then
	echo "The branch name of ${projectName} must be specified."
	exit 1
fi

if [[ -z "${keyId}" ]]
then
	echo "The ID for the signing key must be specified."
	exit 1
fi

# Program Execution
#___________________________________________________________________________________________________
cd "$path/.."
version=$(grep VERSION CMakeLists.txt | tail -1 | awk '{ print $2 }')

if [[ -z "${version}" ]]
then
	echo "Failed to determine the version of ${projectName}."
	exit 1
fi
tarballFileName="${projectName}-${version}.tar"
tarballOrigFileName="${projectName}_${version}.orig.tar.xz"
echo "Generating project tarball..."
hg pull --rev "${branch}"
hg update --clean --rev "${branch}"
hg archive "${tarballFileName}" --rev "${branch}"

if [[ ! -e "${debBuildDirectoryPath}/" ]]
then
	mkdir "${debBuildDirectoryPath}/" || exit $?
fi
mv "${tarballFileName}" "${debBuildDirectoryPath}/" || exit $?
echo "Generating Debian file structure..."
cd "${debBuildDirectoryPath}" || exit $?
tar -xf "${tarballFileName}" || exit $?
cd "${projectName}-${version}/" || exit $?

if [[ -e "${debBuildDirectoryPath}/${projectName}-${version}/pkg/debian" ]]
then
	mv "${debBuildDirectoryPath}/${projectName}-${version}/pkg/debian" . || exit $?
	xz "../${tarballFileName}" || exit $?
	mv "../${tarballFileName}.xz" "../${tarballOrigFileName}" || exit $?
else
	dh_make \
		--yes  \
		--copyright ${licence}  \
		--${packageclass}  \
		--createorig  \
		|| exit $?
fi
echo "Building DEB package..."
dpkg-buildpackage -us -uc || exit $?
echo "Signing DEB package..."
debsign -k${keyId} || exit $?
echo "Cleaning up temporary build files..."
cd .. || exit $?
rm --recursive --dir "${projectName}-${version}/" || exit $?

if [[ -e "${tarballFileName}" ]]
then
	rm "${tarballFileName}" || exit $?
fi
echo "Successfully created DEB package."

